//+------------------------------------------------------------------+
//|                                                    Squize_MA.mq4 |
//|                                                          Kalenzo |
//|                                      bartlomiej.gorski@gmail.com |
//| histo by mrtools                                                 |
//+------------------------------------------------------------------+
#property copyright "Kalenzo"
#property link      "bartlomiej.gorski@gmail.com"
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  MidnightBlue
#property indicator_color2  Maroon
#property indicator_color3  Gray
#property indicator_width1  4
#property indicator_width2  4
#property indicator_width3  5
#property indicator_minimum 0
#property indicator_maximum 1

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    MaDifrential    = 2;
extern int    Ma1Type         = 1;
extern int    Ma1Price        = PRICE_CLOSE;
extern int    Ma1Period       = 3;
extern int    Ma2Type         = 1;
extern int    Ma2Price        = PRICE_CLOSE;
extern int    Ma2Period       = 13;
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsNotify    = true;
extern bool   alertsEmail     = true;
extern string soundFile       = "alert2.wav"; 

//
//
//
//
//

double upma[];
double dnma[];
double state[];
double UpH[];
double DnH[]; 
double NuH[]; 
int    timeFrame;  

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,UpH); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,DnH); SetIndexStyle(1,DRAW_HISTOGRAM);   
   SetIndexBuffer(2,NuH); SetIndexStyle(2,DRAW_HISTOGRAM);     
   SetIndexBuffer(3,upma);
   SetIndexBuffer(4,dnma);
   SetIndexBuffer(5,state);
   
   timeFrame = stringToTimeFrame(TimeFrame);
   IndicatorShortName(timeFrameToString(timeFrame)+" SquizeMA");
return(0);
}

int deinit()
{
return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit=MathMax(Bars-counted_bars,3*timeFrame/Period());

   //
   //
   //
   //
   //
   
   for(int i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
      double treshold = MaDifrential*Point; if (Digits==3 || Digits==5) treshold *= 10.0;
      double ma1   = iMA(NULL,timeFrame,Ma1Period,0,Ma1Type,Ma1Price,y);
      double ma2   = iMA(NULL,timeFrame,Ma2Period,0,Ma2Type,Ma2Price,y); 
      double madif = MathAbs(ma1-ma2);
      
      state[i] = state[i+1];      
         if(madif > treshold)
         {
            upma[i] = ma1;
            dnma[i] = ma2;
            if (ma1>ma2)
                  state[i] =  1;
            else  state[i] = -1;
         }
         else
         {
            state[i] = 0;
            if(ma1>ma2)
            {
               upma[i] = ma1 - (madif*0.5);
               dnma[i] = ma1 - (madif*0.5);
            }
            else
            {
               upma[i] = ma2 - (madif*0.5);
               dnma[i] = ma2 - (madif*0.5);
            }
         }
         
         //
         //
         //
         //
         //
         
         UpH[i]   = EMPTY_VALUE;
         DnH[i]   = EMPTY_VALUE;
         NuH[i]   = EMPTY_VALUE;
         if (state[i] == 1) UpH[i] = 1;
         if (state[i] ==-1) DnH[i] = 1;
         if (state[i] == 0) NuH[i] = 1;
              
   }
   manageAlerts();
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (state[whichBar] != state[whichBar+1])
      {
         if (state[whichBar] == 1)                          doAlert(whichBar,"up");
         if (state[whichBar] ==-1)                          doAlert(whichBar,"down");
         if (state[whichBar] == 0 && state[whichBar+1]== 1) doAlert(whichBar,"from up into flat");
         if (state[whichBar] ==-0 && state[whichBar+1]==-1) doAlert(whichBar,"from down into flat");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

           message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Ma Squize ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," Ma Squize " +" "+message));
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(), Period(), " Ma Squize "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}





